
local AddOnName = "AutoProfitX";
local DBName = "AutoProfitXDB";
local Version = GetAddOnMetadata( AddOnName, "Version" );

-- Create our addon.
AutoProfitX = LibStub("AceAddon-3.0"):NewAddon( AddOnName, 
                                              "AceConsole-3.0", 
                                              "AceEvent-3.0" );

local L = LibStub("AceLocale-3.0"):GetLocale( AddOnName, false );

local AceGUI = LibStub("AceGUI-3.0")

local LDB = LibStub("LibDataBroker-1.1", true)
local LDBIcon = LDB and LibStub("LibDBIcon-1.0", true)

local tooltip = LibStub("LibGratuity-3.0", true)

_G["apxAuthor"] = {
     ["Lindarena@Laughing Skull"] = true,
     ["Nagdand@Laughing Skull"] = true,
     ["Nagdand@Korialstrasz"] = true,
     ["Holytree@Laughing Skull"] = true,
     ["Loansone@Laughing Skull"] = true,
}

AutoProfitXButtonSpinner = {
     { name = L["APX_MISC_NEVER"], value = 0, },
     { name = L["APX_MISC_MOUSEOVER_PROFIT"], value = 1, },
     { name = L["APX_MISC_MOUSEOVER"], value = 2, },
     { name = L["APX_MISC_PROFIT"], value = 3, },
}

AutoProfitXArmorSub = {
     DEATHKNIGTH = [=[

     |cffc41f3bDeath Knight|r :
          Mail
          Leather
          Cloth
     ]=],
     PALADIN = [=[

     |cfff58cba%s|r :
          Mail
          Leather
          Cloth
     ]=],
     WARRIOR = [=[

     |cffc79c6eWarrior|r :
          Mail
          Leather
          Cloth
     ]=],
     HUNTER = [=[

     |cffabd473Hunter|r :
          Leather
          Cloth
     ]=],
     SHAMAN = [=[

     |cff0070deShaman|r :
          Cloth
     ]=],
     DRUID = [=[

     |cffff7d0aDruid|r :
          Cloth
     ]=],
     ROGUE = [=[

     |cfffff569Rogue|r :
          Cloth
     ]=],
}

AutoProfitXGenderTable = {
     "Unknown",
     "MALE",
     "FEMALE",
}

function AutoProfitX:GetLocalClass()
     local sex = UnitSex( "player" )
     local class = select( 2, UnitClass( "player" ) )
     return _G["LOCALIZED_CLASS_NAMES_"..AutoProfitXGenderTable[sex]][class]
end

local apxf = CreateFrame( "Frame", "ProfitFrame", UIParent )
local name, realm
local totalProfit = 0
--make proficiency table local (defined in proficiencies.lua)
local prof = AutoProfitX_Proficiencies[ select( 2, UnitClass( "player" ) ) ] or {}
AutoProfitX_Proficiencies = nil

--some strings commonly used in addon
local linkMatch = L["APX_FORMATED_LM"];
local classMatch = string.format( L["APX_FORMATED_ICA"], "([%w, ]*)" )
local equipMatch = string.format( L["APX_FORMATED_IES"], "([%w, ]*)" )

local sCmds = {
     "autoprofitx",
     "autoprofit",
     "apx",
}

local sOpts = {
     name = L["APX_SLASH_LABEL"],
     type = "group",
     handler = AutoProfitX,
     args = {
          Add = {
               type = "execute", order = 1,
               name = L["APX_SLASH_ADD"],
               desc = L["APX_SLASH_ADD_DESC"],
               func = function( self )
		    local str = self.input
		    local sgsub = string.gsub( str, "add ", "")
                    local slocal = string.find( str, "local" )
                    local sglobal = string.find( str, "global" )
                    local tbl = string.match( sgsub, AutoProfitX.db.global.regx )
                    if ( not tbl ) then
                         AutoProfitX:Print( "error", L["APX_SLASH_ADD_DESC"] )
                         return;
                    end
                    if ( slocal ) then
                         typ = "local"
                    elseif  ( sglobal ) then
                         typ = "global"
                    end
                    AutoProfitX:idUtils( "idAdd", typ, tbl );
               end,
          },
          Remove = {
               type = "execute", order = 2,
               name = L["APX_SLASH_REMOVE"],
               desc = L["APX_SLASH_REMOVE_DESC"],
               func = function( self )
                    local str = self.input
                    local sgsub = string.gsub( str, "remove ", "")
                    local slocal = string.find( str, "local" )
                    local sglobal = string.find( str, "global" )
                    local tbl = string.match( sgsub, AutoProfitX.db.global.regx )
                    if ( not tbl ) then
                         AutoProfitX:Print( "error", L["APX_SLASH_REMOVE_DESC"] )
                         return;
                    end
                    if ( slocal ) then
                         typ = "local"
                    elseif  ( sglobal ) then
                         typ = "global"
                    end
                    AutoProfitX:idUtils( "idRemove", typ, tbl );
               end,
          },
          List = {
               type = "execute", order = 3,
               name = L["APX_SLASH_LIST"],
               desc = L["APX_SLASH_LIST_DESC"],
               func = function( self )
                    local action, lType = string.match( self.input, "(.*)%s(.*)" )
                    if ( not lType ) then 
                         AutoProfitX:Print( "error", L["APX_SLASH_LIST_DESC"] );
                         return;
                    end
                    AutoProfitX:ListExceptions( lType );
               end
          },
          excep = {
               type = "execute", order = 4,
               name = L["APX_SLASH_EXCEPTIONS"],
               desc = L["APX_SLASH_EXCEPTIONS_DESC"],
               func = function()
                    if ( not AutoProfitXFrame:IsVisible() ) then
                         AutoProfitX:Print( "bug", "Show AutoProfitX Frame");
                         AutoProfitXFrame:Show();
                    else
                         AutoProfitX:Print( "bug", "Hideing AutoProfitX Frame");
                         AutoProfitXFrame:Hide();
                    end
               end,
          },
          options = {
               type = "execute", order = 5,
               name = L["APX_SLASH_OPTIONS"],
               desc = L["APX_SLASH_OPTIONS_DESC"],
               func = function()
                  InterfaceOptionsFrame_OpenToCategory( AutoProfitX.ConfigFrames.AutoProfitX );
               end,
          },
     },
}

local apx = {
     type = "group",
     name = L["APX_ABOUT_LABEL"],
     args = {
          confgendesc = {
               type = "description", order = 1,
               name = L["APX_ABOUT_ABOUT_DESC"],
               cmdHidden = true,
          },
          cat1 = {
               name = L["APX_ABOUT_ABOUT"],
               type = "group", order = 2, inline = true,
               args = {
                    confversiondesc = {
                         type = "description", order = 1,
                         name = L["APX_ABOUT_VERSION"]:format( Version ),
                         cmdHidden = true,
                    },
                    confauthordesc = {
                         type = "description", order = 2,
                         name = L["APX_ABOUT_AUTHOR"]:format( GetAddOnMetadata("AutoProfitX", "Author") ),
                         cmdHidden = true,
                    },
                    confcatdesc = {
                         type = "description", order = 3,
                         name = L["APX_ABOUT_CATEGORY"]:format( GetAddOnMetadata("AutoProfitX", "X-Category") ),
                         cmdHidden = true,
                    },
                    confemaildesc = {
                         type = "description", order = 4,
                         name = L["APX_ABOUT_EMAIL"]:format( GetAddOnMetadata("AutoProfitX", "X-Email") ),
                         cmdHidden = true,
                    },
                    confwebsitedesc = {
                         type = "description", order = 5,
                         name = L["APX_ABOUT_WEBSITE"]:format( GetAddOnMetadata("AutoProfitX", "X-Website") ),
                         cmdHidden = true,
                    },
               },
          },
          cat2 = {
               type = "group", order = 3,
               name = L["APX_OPTIONS_LABEL"],
               args = {
                    loadmsg = {
                         type = "toggle", order = 1,
                         name = L["APX_ABOUT_LOAD"],
                         desc = L["APX_ABOUT_LOAD_DESC"],
                         get = function() return AutoProfitX.db.global.showGreeting; end,
                         set = function( _, value )
                              if ( value ) then
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_ABOUT_LOAD"], L["APX_OPTIONS_ON"] ) );
                              else
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_ABOUT_LOAD"], L["APX_OPTIONS_OFF"] ) );
                              end
                              AutoProfitX.db.global.showGreeting = value; 
                         end,
                    },
                    memusage = {
                         type = "execute", order = 2,
                         name = L["APX_ABOUT_MEMORY"],
                         desc = L["APX_ABOUT_MEMORY_DESC"],
                         func = function()
                              UpdateAddOnMemoryUsage();
                              AutoProfitX:Print( "print", L["APX_ABOUT_MEMORY_FORMATED"]:format( floor(GetAddOnMemoryUsage(AddOnName)) ) )
                         end,
                    },
                    dockButton = {
                         name = L["APX_OPTIONS_RESET"],
                         type = "execute", order = 2,
                         desc = L["APX_OPTIONS_RESET_DESC"],
                         func = function ()
                              AutoProfitX:Print( "print", L["APX_OPTIONS_RESET_PRINT"] )
                              AutoProfitX:SetButtonXY( -41, -37 ) 
                         end,
                    },
               },
          },
          cat3 = {
               type = "group", order = 4,
               name = L["APX_OPTIONS_SELL_LABEL"],
               args = {
                    autoSell = {
                         name = L["APX_OPTIONS_AUTOSELL"],
                         type = "toggle", order = 1,
                         desc = L["APX_OPTIONS_AUTOSELL_DESC"],
                         get = function () return AutoProfitX.db.char.autoSell;  end,
                         set = function( _, value )
                              if ( value ) then
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_AUTOSELL"], L["APX_OPTIONS_ON"] ) );
                              else
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_AUTOSELL"], L["APX_OPTIONS_OFF"] ) );
                              end
                              AutoProfitX.db.char.autoSell = value;
                         end,
                    },
                    sellSoulbound = {
                         name = L["APX_OPTIONS_SOULBOUND"],
                         type = "toggle", order = 2,
                         desc = L["APX_OPTIONS_SOULBOUND_DESC"],
                         get = function () return AutoProfitX.db.char.checkSoulbound; end,
                         set = function ( _, value )
                              if ( value ) then
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_SOULBOUND"], L["APX_OPTIONS_ON"] ) );
                              else
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_SOULBOUND"], L["APX_OPTIONS_OFF"] ) );
                              end
                              AutoProfitX.db.char.checkSoulbound = value;
                         end,
                    },
                    openBags = {
                         name = L["APX_OPTIONS_BAG"],
                         type = "toggle", order = 3,
                         desc = L["APX_OPTIONS_BAG_DESC"],
                         get = function () return AutoProfitX.db.char.openBags; end,
                         set = function ( _, value )
                              local bagCheck = AutoProfitX:otherBag()
                              if ( not bagCheck ) then
                                   if ( value ) then
                                        AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_BAG"], L["APX_OPTIONS_ON"] ) );
                                   else
                                        AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_BAG"], L["APX_OPTIONS_OFF"] ) );
                                   end
                                   AutoProfitX.db.char.openBags = value;
                              else
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_BAG_ADDON"] );
                              end
                         end,
                    },
                    sellarmorsub = {
                         name = L["APX_OPTIONS_SELLARMORSUB"],
                         type = "toggle", order = 4,
                         desc = function()
                              local class = AutoProfitX:GetLocalClass();
                              local ArmorSub = AutoProfitXArmorSub[ select( 2, UnitClass( "player" ) ) ];
			      local formated = ArmorSub:format( class );
	                      return L["APX_OPTIONS_SELLARMORSUB_DESC"]:format( formated );
                              --desc = L["APX_OPTIONS_SELLARMORSUB_DESC"]:format( AutoProfitXArmorSub[ select( 2, UnitClass( "player" ) ) ] ),
                         end,
                         get = function() return AutoProfitX.db.char.sellarmorsub; end,
                         set = function( _, value )
                              if ( value ) then
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_SELLARMORSUB"], L["APX_OPTIONS_ON"] ) );
                              else
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_SELLARMORSUB"], L["APX_OPTIONS_OFF"] ) );
                              end
                              AutoProfitX.db.char.sellarmorsub = value;
                         end
                    },
               },
          },
          cat4 = {
               type = "group", order = 5,
               name = L["APX_OPTIONS_SALES_LABEL"],
               args = {
                    silent = {
                         name = L["APX_OPTIONS_SALES"],
                         type = "toggle", order = 1,
                         desc = L["APX_OPTIONS_SALES_DESC"],
                         get = function () return AutoProfitX.db.char.silent; end,
                         set = function ( _, value )
                              if ( value ) then
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_SALES"], L["APX_OPTIONS_ON"] ) );
                              else
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_SALES"], L["APX_OPTIONS_OFF"] ) );
                              end
                              AutoProfitX.db.char.silent = value;
                         end,
                    },
                    fancysales = {
                         name = L["APX_OPTIONS_FANCY"],
                         type = "toggle", order = 2,
                         desc = L["APX_OPTIONS_FANCY_DESC"],
                         get = function () return AutoProfitX.db.char.fancysales; end,
                         set = function ( _, value )
                              if ( value ) then
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_FANCY"], L["APX_OPTIONS_ON"] ) );
                              else
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_FANCY"], L["APX_OPTIONS_OFF"] ) );
                              end
                              AutoProfitX.db.char.fancysales = value; end,
                    },
                    stackcount = {
                         name = L["APX_OPTIONS_STACK"],
                         type = "toggle", order = 3,
                         desc = L["APX_OPTIONS_STACK_DESC"],
                         get = function () return AutoProfitX.db.char.stackCount; end,
                         set = function ( _, value )
                              if ( value ) then
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_STACK"], L["APX_OPTIONS_ON"] ) );
                              else
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_STACK"], L["APX_OPTIONS_OFF"] ) );
                              end
                              AutoProfitX.db.char.stackCount = value;
                         end,
                    },
               },
          },
          cat5 = {
               type = "group", order = 6,
               name = L["APX_OPTIONS_PROFIT_LABEL"],
	       args = {
                    profit = {
                         name = L["APX_OPTIONS_PROFIT"],
                         type = "toggle", order = 1,
                         desc = L["APX_OPTIONS_PROFIT_DESC"],
                         get = function () return AutoProfitX.db.char.showTotal; end,
                         set = function ( _, value )
                              if ( value ) then
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_PROFIT"], L["APX_OPTIONS_ON"] ) );
                              else
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_PROFIT"], L["APX_OPTIONS_OFF"] ) );
                              end
                              AutoProfitX.db.char.showTotal = value;
                         end,
                    },
                    fancyprofits = {
                         name = L["APX_OPTIONS_FANCY_PROFIT"],
                         type = "toggle", order = 2,
                         desc = L["APX_OPTIONS_FANCY_PROFIT_DESC"],
                         get = function () return AutoProfitX.db.char.fancyprofits; end,
                         set = function ( _, value )
                              if ( value ) then
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_FANCY_PROFIT"], L["APX_OPTIONS_ON"] ) );
                              else
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_OPTIONS_FANCY_PROFIT"], L["APX_OPTIONS_OFF"] ) );
                              end
                              AutoProfitX.db.char.fancyprofits = value;
                         end,
                    },
                    buttonSpinRate = {
                         name = L["APX_OPTIONS_GOLDPILE"],
                         type = "range", order = 3,
                         desc = L["APX_OPTIONS_GOLDPILE_DESC"],
                         min = 0,
                         max = 1,
                         step = .1,
                         get = function() return AutoProfitX.db.char.spinRate; end,
                         set = function( _, value )
                              AutoProfitX_SellButton_TreasureModel.rotRate = value;
                              AutoProfitX.db.char.spinRate = value;
                         end,
                    },
                    buttonSpin = {
                         name = L["APX_OPTIONS_BUTTONA"],
                         type = "select", order = 4,
                         desc = L["APX_OPTIONS_BUTTONA_DESC"],
                         get = function() return AutoProfitX.db.char.buttonSpin; end,
                         set = function( _, value ) AutoProfitX.db.char.buttonSpin = value; end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( AutoProfitXButtonSpinner ) do
                                   if ( v.value ~= AutoProfitX.db.char.buttonSpin ) then
                                        preList[v.value] = v.name
                                   else
                                        preList[v.value] = "|cffffff9a"..v.name.."|r"
                                   end
                              end
                              return preList;
                         end
                    },
               },
          },
          cat6 = {
               type = "group", order = 7,
               name = L["APX_ABOUT_MINIMAP_ICON"],
               args = {
                    hidemini = {
                         type = "toggle", order = 1,
                         name = L["APX_ABOUT_MINIMAP"],
                         desc = L["APX_ABOUT_MINIMAP_DESC"],
                         get = function() return AutoProfitX.db.global.apxb.hide; end,
                         set = function( _, value )
                              if ( not value ) then
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_ABOUT_MINIMAP_ICON"], L["APX_OPTIONS_ON"] ) );
                                   LDBIcon:Show( AddOnName );
                              else
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_ABOUT_MINIMAP_ICON"], L["APX_OPTIONS_OFF"] ) );
                                   LDBIcon:Hide( AddOnName );
                              end
                              AutoProfitX.db.global.apxb.hide = value; 
                         end,
                    },
               },
          },
     },
}

local apxDebuging = {
     name = L["APX_DEBUG_LABEL"],
     type = "group",
     args = {
          confinfodesc = {
               name = L["APX_DEBUG_SETTINGS"],
               type = "group", inline = true,
               order = 3,
               args = {
                    confgendesc = {
                         type = "description",
                         name = L["APX_DEBUG_DESC"],
                         cmdHidden = true,
                         order = 1,
                    },
                    optdebug = {
                         type = "toggle",
                         width = "full",
                         name = L["APX_DEBUG_OPTDEBUG"],
                         desc = L["APX_DEBUG_OPTDEBUG_DESC"],
                         get = function() return AutoProfitX.db.global.debug.state; end,
                         set = function( _, value )
                              if ( value ) then
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_DEBUG_PRINT"], L["APX_OPTIONS_ON"] ) );
                              else
                                   AutoProfitX:Print( "print", L["APX_OPTIONS_TOGGLE"]:format( L["APX_DEBUG_PRINT"], L["APX_OPTIONS_OFF"] ) );
                              end
                              AutoProfitX.db.global.debug.state = value; 
                         end,
                         order = 4,
                    },
                    regx = {
                         type = "input",
                         name = "RegEx Pattren",
                         desc = "for string functions",
                         get = function() return AutoProfitX.db.global.regx; end,
                         set = function( _, value ) AutoProfitX.db.global.regx = value; end,
                    },
               },
          },
     },
}

local Defaults = {
     global = {
          ["geList"] = {
          },
          ["showGreeting"] = true,
          ["apxb"] = {
               ["hide"] = false,
          },
          ["debug"] = {
               ["state"] = false,
          },
          ["regx"] = "[a-zA-Z]%s\|(.*)",
          ["temp"] = {
          },
          ["otherBag"] = false,
	  ["Version"] = false,
          ["bag"] = {
               [0] = false,
               [1] = false,
               [2] = false,
               [3] = false,
               [4] = false,
          },
     },
     char = {
          ["autoSell"] = false,
          ["silent"] = false,
          ["fancysals"] = false,
          ["showTotal"] = true,
          ["buttonYpos"] = -37,
          ["buttonXpos"] = -41,
          ["checkSoulbound"] = false,
          ["buttonSpin"] = 3,
          ["spinRate"] = 0.6,
          ["fancyprofits"] = false,
          ["stackCount"] = false,
          ["openBags"] = false,
          ["sellarmorsub"] = false,
          ["leList"] = {
          },
     },
}

-- AutoProfitX:OnInitialize()
-- =========================================================
function AutoProfitX:OnInitialize()
     -- Set up our database
     self.db = LibStub("AceDB-3.0"):New( DBName, Defaults, true );

     -- Set up our config options.
     local profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable( self.db );
     local AutoProfitXConf = LibStub("AceConfig-3.0");
     local AutoProfitXReg  = LibStub("AceConfigRegistry-3.0");
     local AutoProfitXDiag = LibStub("AceConfigDialog-3.0");

     name = UnitName("player")
     realm = GetCVar("realmName")
     local curUser = name.."@"..realm;

     AutoProfitXConf:RegisterOptionsTable( AddOnName, sOpts, sCmds );

     AutoProfitXReg:RegisterOptionsTable( AddOnName.." apx", apx );
     if ( _G["apxAuthor"][curUser] ) then AutoProfitXReg:RegisterOptionsTable( AddOnName.." Debuging", apxDebuging); end

     self.ConfigFrames = { AutoProfitX = AutoProfitXDiag:AddToBlizOptions( AddOnName.." apx", AddOnName ), }

     if ( _G["apxAuthor"][curUser] ) then self.ConfigFrames.Debuging = AutoProfitXDiag:AddToBlizOptions( AddOnName.." Debuging", "Debuging", AddOnName ); end

     self:RegisterEvent("PLAYER_LOGIN");
     self:RegisterEvent("PLAYER_ENTERING_WORLD");

     self.apxf = apxf;

     if ( not AutoProfitX.db.global.Version or AutoProfitX.db.global.Version ~= Version ) then
          AutoProfitX:Print( "bug", "Resetting Data Aray" )
          -- wipe( AutoProfitX.db.global )
	  -- wipe( AutoProfitX.db.char )
          AutoProfitX.db.global.Version = Version;
     end

     AutoProfitX:ExceptionsFrame();
end

-- AutoProftX:DropDownMenuOnLoad()
-- =========================================================
function AutoProfitX:DropDownMenuOnLoad()
     UIDropDownMenu_Initialize( AutoProfitXDropDownMenu, DropDownMenu_Initialize, "MENU", 1);
end

-- DropDownMenu_Initialize( frame, level )
-- =========================================================
function DropDownMenu_Initialize( frame, level )
     local apxInfo
     apxInfo = UIDropDownMenu_CreateInfo();
     apxInfo["text"] = AddOnName.." "..L["APX_OPTIONS_LABEL"];
     apxInfo["isTitle"] = true;
     apxInfo["notCheckable"] = true;
     UIDropDownMenu_AddButton( apxInfo );

     apxInfo = UIDropDownMenu_CreateInfo();
     apxInfo["text"] = "";
     apxInfo["isTitle"] = true;
     apxInfo["notCheckable"] = true;
     UIDropDownMenu_AddButton( apxInfo );

     apxInfo = UIDropDownMenu_CreateInfo();
     apxInfo["text"] = L["APX_OPTIONS_LABEL"];
     apxInfo["checked"] = nil;
     apxInfo["func"] = function() InterfaceOptionsFrame_OpenToCategory( AutoProfitX.ConfigFrames.AutoProfitX ); end;
     apxInfo["icon"] = nil;
     apxInfo["arg1"] = "" .. 1;
     apxInfo["tCoordLeft"] = 0;
     apxInfo["tCoordRight"] = 1;
     apxInfo["tCoordTop"] = 0;
     apxInfo["tCoordBottom"] = 1;
     apxInfo["notCheckable"] = true;
     UIDropDownMenu_AddButton( apxInfo );

     name = UnitName("player")
     realm = GetCVar("realmName")
     local curUser = name.."@"..realm;
     if ( _G["apxAuthor"][curUser] ) then 

     apxInfo = UIDropDownMenu_CreateInfo();
     apxInfo["text"] = "";
     apxInfo["isTitle"] = true;
     apxInfo["notCheckable"] = true;
     UIDropDownMenu_AddButton( apxInfo );

     apxInfo = UIDropDownMenu_CreateInfo();
     apxInfo["text"] = L["APX_DEBUG_OPTDEBUG"];
     apxInfo["checked"] = nil;
     apxInfo["func"] = function() InterfaceOptionsFrame_OpenToCategory( AutoProfitX.ConfigFrames.Debuging ); end;
     apxInfo["icon"] = nil;
     apxInfo["arg1"] = "" .. 1;
     apxInfo["tCoordLeft"] = 0;
     apxInfo["tCoordRight"] = 1;
     apxInfo["tCoordTop"] = 0;
     apxInfo["tCoordBottom"] = 1;
     apxInfo["notCheckable"] = true;
     UIDropDownMenu_AddButton( apxInfo );
     end
end

-- AutoProfitX:OnEnable()
-- =========================================================
function AutoProfitX:OnEnable()
     self:RegisterEvent("MERCHANT_SHOW");
     self:RegisterEvent("BAG_UPDATE");
     self:RegisterEvent("PLAYER_LOGIN");
     self:RegisterEvent("PLAYER_ENTERING_WORLD");
end

-- AutoProfitX:OnDisable()
-- =========================================================
function AutoProfitX:OnDisable()
     self:UnregisterEvent("MERCHANT_SHOW");
     self:UnregisterEvent("BAG_UPDATE");
     self:UnregisterEvent("PLAYER_LOGIN");
     self:UnregisterEvent("PLAYER_ENTERING_WORLD");
end

-- AutoProfitX:MERCHANT_SHOW()
-- =========================================================
function AutoProfitX:MERCHANT_SHOW()
     if ( AutoProfitX.db.char.openBags ) then
          AutoProfitX:Print( "bug", "Opening All Bags" );
          AutoProfitX:OpenAllBags( false )
     end
     if ( AutoProfitX.db.char.autoSell ) then
          AutoProfitX:Print( "bug", "autoSell is on" );
          local profit = AutoProfitX:GetProfit()
          if ( profit > 0 ) then
               AutoProfitX:SellJunk()
               if ( AutoProfitX.db.char.showTotal ) then
                    if ( AutoProfitX.db.char.fancyprofits ) then 
                         AutoProfitX:Print( "print", format( L["APX_FORMATED_PROFITS"], AutoProfitX:GetMoneyString( profit ) ) );
                    else
                         AutoProfitX:Print( "print", format( L["APX_FORMATED_PROFITS"], AutoProfitX:CopperToGold( profit ) ) );
                    end
               end
          end
     else
          -- register BAG_UPDATE event for updating button
          AutoProfitX.apxf:RegisterEvent("BAG_UPDATE")
          AutoProfitX.apxf:RegisterEvent("MERCHANT_CLOSED")
     end
end

-- AutoProfitX:BAG_UPDATE()
-- =========================================================
function AutoProfitX:BAG_UPDATE()
     if ( AutoProfitX_SellButton:IsVisible() ) then
          AutoProfitX:OnShowButton()
     end
end

-- AutoProfitX:PLAYER_LOGIN()
-- =========================================================
function AutoProfitX:PLAYER_LOGIN()
     -- LDB launcher
     if ( LDB ) then
          AutoProfitXLauncher = LDB:NewDataObject( "AutoProfitX", {
               type = "launcher",
               icon = "Interface\\Icons\\INV_Misc_Bag_EnchantedMageweave",
               OnClick = function( clickedframe, button )
                    if ( button == "RightButton" ) then
                         ToggleDropDownMenu( 1, nil, AutoProfitXDropDownMenu, clickedframe:GetName(), 0, -5 )
                         -- AutoProfitX:ButtonClick( "config" );
                    else
                         AutoProfitX:ButtonClick( "list" )
                    end
               end,
               OnTooltipShow = function( tt )
                    tt:AddLine( L["APX_MINIMAP_TEXT1"]:format( Version ) );
                    tt:AddLine( L["APX_MINIMAP_TEXT2"] );
                    tt:AddLine( L["APX_MINIMAP_TEXT3"] );
               end,
          } )
          if ( LDBIcon and not IsAddOnLoaded( "Broker2FuBar" ) and not IsAddOnLoaded( "FuBar") ) then
               LDBIcon:Register( "AutoProfitX", AutoProfitXLauncher, AutoProfitX.db.global.apxb );
          end
     end
end

-- AutoProfitX:PLAYER_ENTERING_WORLD()
-- =========================================================
function AutoProfitX:PLAYER_ENTERING_WORLD()
     local bagCheck = AutoProfitX:otherBag()
     if ( bagCheck ) then AutoProfitX.db.char.openBags = false; end
     AutoProfitX_SellButton:SetPoint( "TOPRIGHT", "MerchantFrame", "TOPRIGHT", AutoProfitX.db.char.buttonXpos, AutoProfitX.db.char.buttonYpos );
end

-- AutoProfitX:MERCHANT_CLOSED()
-- =========================================================
function AutoProfitX:MERCHANT_CLOSED()
     if ( AutoProfitX.db.char.openBags ) then
          AutoProfitX:Print( "bug", "Closing All Bags" );
          AutoProfitX:OpenAllBags( true )
     end
     -- unregister BAG_UPDATE event used for updating icon
     -- --------------------------------------------------
     if ( AutoProfitX.apxf:IsEventRegistered("BAG_UPDATE") ) then
          AutoProfitX.apxf:UnregisterEvent("MERCHANT_CLOSED")
          AutoProfitX.apxf:UnregisterEvent("BAG_UPDATE")
     end
end

-- AutoProfitX:idUtils( )
-- =========================================================
function AutoProfitX:idUtils( action, type, exceptions )
     local itemID, name
     if ( action == "idCheck" ) then
          itemID = AutoProfitX:GetID( link )
          if ( AutoProditX.db.global.geList[itemID] ) then return true; end
          if ( AutoProfitX.db.char.leList[itemID] ) then return true; end
          return false;
     elseif ( action == "idRemove" ) then
          for link in string.gmatch( exceptions, L["APX_FORMATED_LM"] ) do
               name = GetItemInfo( link );
	       itemID = AutoProfitX:GetID( link )
               if ( itemID ) then
                    if ( type == "global") then
                         AutoProfitX:Print( "print", format( L["APX_FORMATED_REMOVE"], link, "Global" ) )
                         AutoProfitX.db.global.geList[itemID] = nil;
                    elseif( type == "local" ) then
                         AutoProfitX:Print( "print", format( L["APX_FORMATED_REMOVE"], link, "Character" ) )
                         AutoProfitX.db.char.leList[itemID] = nil;
                    end
               else
                    AutoProfitX:Print( "print", L["APX_OPTIONS_BADLINK"] )
               end
          end
     elseif ( action == "idAdd" ) then
          for link in string.gmatch( exceptions, L["APX_FORMATED_LM"] ) do
               name = GetItemInfo( link );
	       itemID = AutoProfitX:GetID( link )
               if ( itemID ) then
                    if ( type == "global") then
	                 AutoProfitX:Print( "print", format( L["APX_FORMATED_ADD"], link, "Global" ) )
                         AutoProfitX.db.global.geList[itemID] = link;
                    elseif( type == "local" ) then
                         AutoProfitX:Print( "print", format( L["APX_FORMATED_ADD"],  link, "Character" ) )
                         AutoProfitX.db.char.leList[itemID] = link;
                    end
               else
                    AutoProfitX:Print( "print", L["APX_OPTIONS_BADLINK"] )
               end
          end
     end
     if ( AutoProfitX_SellButton:IsVisible() ) then
          AutoProfitX:OnShowButton()
     end
end

-- AutoProfitX:ListExceptions()
-- =========================================================
function AutoProfitX:ListExceptions( lType )
     local eList
     local uType = string.upper( lType );
     if ( uType == "GLOBAL" ) then
          eList = AutoProfitX.db.global.geList;
     elseif ( uType == "LOCAL" ) then
          eList = AutoProfitX.db.char.leList;
     end
     local link
     local dispHeader = true
     for i in pairs( eList ) do
          if ( dispHeader ) then
               self:Print( "print", L["APX_EXCEPTION_"..uType] );
               dispHeader = false
          end
          _, link = GetItemInfo( i )
          if ( link ) then
               self:Print( "print", link );
          end
     end
     if ( dispHeader ) then
          self:Print( "print", L["APX_EXCEPTION_EMPTY_"..uType] );
     end
end

--[[ AutoProfitX:ImportExceptionList()
-- =========================================================
function AutoProfitX:ImportExceptionList( fromChar )
     local iName, iRealm = strsplit("@",fromChar)
     if ( iName ) then
          if ( self.db.char[iRealm] and self.db.char[iRealm][iName] and self.db.char[iRealm][iName].exceptionList ) then
               eList = tdeepcopy(self.db.char[iRealm][iName].exceptionList)
               self.db.char[realm][name].exceptionList = eList
               self:Print( "print", format( L["Exception list imported from %s on %s."], iName, iRealm ) )
          else
               self:Print( "print", format( L["Exception list could not be found for %s on %s."], iName, iRealm ) )
          end
          return
     end
end --]]

-- AutoProfitX:GetID( link )
-- =========================================================
function AutoProfitX:GetID( link )
     return string.match( link, "item:(%d+)" )
end

-- AutoProfitX:SellJunk()
-- =========================================================
function AutoProfitX:SellJunk()
     local link, money
     if ( MerchantFrame:IsVisible() and MerchantFrame.selectedTab == 1 ) then
          for bag = 0, 4 do
               for slot = 1, GetContainerNumSlots(bag) do
                    --if slot not empty and item is junk
                    link = GetContainerItemLink(bag, slot)
                    if ( link and self:IsJunk( bag, slot, link ) ) then
                         --sell item
                         UseContainerItem( bag, slot )
                         --if sale reporting is turned on (not silent), display sale message
                         if ( AutoProfitX.db.char.silent ) then
                              money = select( 11, GetItemInfo( link ) ) * select( 2, GetContainerItemInfo( bag, slot ) )
                              if ( AutoProfitX.db.char.fancysales ) then
                                   if ( AutoProfitX.db.char.stackCount ) then
                                        self:Print( "print",  format( L["APX_FORMATED_COINSTACK"],  link, GetItemCount( link ), self:GetMoneyString( money ) ) )
                                   else
                                        self:Print( "print", format( L["APX_FORMATED_COIN"], link, self:GetMoneyString( money ) ) )
                                   end
                              else
                                   if ( AutoProfitX.db.char.stackCount ) then
                                        self:Print( "print", format( L["APX_FORMATED_TEXTSTACK"], link, GetItemCount( link ) ) )                                   
                                   else
                                        self:Print( "print", format( L["APX_FORMATED_TEXT"], link ) )
                                   end
                              end
                         end
                    end
               end
          end
     end
end

-- AutoProfitX:IsJunk()
-- =========================================================
function AutoProfitX:IsJunk( bag, slot, link )
     local _, _, quality = GetItemInfo( link )
     local id = AutoProfitX:GetID( link )
     if ( AutoProfitX.db.char.leList[id] or AutoProfitX.db.global.geList[id] ) then
          return quality ~= 0
     elseif ( quality == 0 ) then
          return true;
     elseif ( AutoProfitX.db.char.checkSoulbound and bag and slot ) then
          if ( not AutoProfitX:IsUsable( bag, slot, link ) ) then
               AutoProfitX:Print( "bug", "Soulbound Sellable" )
               return true;
          end
     end
     return false;
end

-- AutoProfitX:IsUsable()
-- =========================================================
function AutoProfitX:IsUsable( bag, slot, link )
     tooltip:SetBagItem( bag, slot );
     if ( not tooltip:Find( ITEM_SOULBOUND ) ) then
          return true; --if it's not soulbound then you can always use it
     end
     if ( tooltip:Find( equipMatch ) ) then
          return true; -- SLAM ON DA BREAKS -- this is part of someone gear set
     end
     local class = UnitClass( "player" ) -- check if item has class requirement
     local _, _, classes = tooltip:Find( classMatch );
     if ( classes ) then
          local found = false
          classes = { strsplit( ", ", classes ) };
          for _, c in ipairs( classes ) do
               if ( string.match( c, class ) ) then
                    found = true
               end
          end
          if ( not found ) then return false; end -- class not in the list so you can't use it
     end
     local _, _, _, _, _, iType, iSubType, _, iEquipLoc = GetItemInfo( link ); -- check if class can ever use this item
     local tbl = prof[iType]
     if ( tbl and ( tbl[iSubType] or ( tbl.noOffhand and iEquipLoc == "INVTYPE_WEAPONOFFHAND" ) ) ) then
          return false;
     end

     -- If this is enabled then all armor sub classes will considered as Unusable
     local class = select( 2, UnitClass( "player" ) )
     if ( AutoProfitX.db.char.sellarmorsub ) then
          if ( ( class == "DEATHKNIGHT" ) and ( iType == "Armor" ) and ( ( iSubType == "Mail" ) or ( iSubType == "Leather" ) or ( iSubType == "Cloth" ) ) ) then
               return false;
          elseif ( ( class == "PALADIN" ) and ( iType == "Armor" ) and ( ( iSubType == "Mail" ) or ( iSubType == "Leather" ) or ( iSubType=="Cloth" ) ) ) then
               return false;
          elseif ( ( class == "WARRIOR" ) and ( iType == "Armor" ) and ( ( iSubType == "Mail" ) or ( iSubType == "Leather" ) or ( iSubType == "Cloth" ) ) ) then
               return false;
          elseif ( ( class == "HUNTER" ) and ( iType == "Armor" ) and ( ( iSubType == "Leather" ) or ( iSubType == "Cloth" ) ) ) then
               return false;
          elseif ( ( class == "SHAMAN" ) and ( iType == "Armor" ) and ( iSubType == "Cloth" ) ) then 
               return false;
          elseif ( ( class == "DRUID" ) and ( iType == "Armor" ) and ( iSubType == "Cloth" ) ) then
               return false;
          elseif ( ( class == "ROGUE" ) and ( iType == "Armor" ) and ( iSubType == "Cloth" ) ) then
               return false;
          end
     end

     -- Seems Usable
     return true;
end

-- AutoProfitX:GetProfit()
-- =========================================================
function AutoProfitX:GetProfit()
     totalProfit = 0
     if ( MerchantFrame:IsVisible() ) then
          local bagSlots, link
          for bag = 0, 4 do
               bagSlots = GetContainerNumSlots( bag )
               if ( bagSlots > 0 ) then
                    for slot = 1, bagSlots do
                         link = GetContainerItemLink( bag, slot )
                         if ( link ) then
                              if ( AutoProfitX:IsJunk( bag, slot, link ) ) then
                                   AutoProfitX_Tooltip:SetBagItem( bag, slot )
                              end
                         end
                    end
               end
          end
     end
     return totalProfit
end

-- AutoProfitX:DragButton()
-- =========================================================
function AutoProfitX:DragButton()
     local MAX_RIGHT = -41
     local MAX_LEFT = -280
     local detatch = false
     local scale = MerchantFrame:GetEffectiveScale()
     local cursorOffset = 1
     local xpos, ypos = GetCursorPosition()
     xpos = xpos/scale
     ypos = ypos/scale
     local mwpos = MerchantFrame:GetRight()
     if ( IsShiftKeyDown() or AutoProfitX.db.char.buttonXpos > MAX_RIGHT or AutoProfitX.db.char.buttonXpos < MAX_LEFT or AutoProfitX.db.char.buttonYpos ~= buttonY ) then
          detatch = true
     end
     xpos = xpos - mwpos - cursorOffset
     if ( detatch ) then
          local mwypos = MerchantFrame:GetTop()
          ypos = ypos - mwypos - cursorOffset
     else
          if ( xpos > MAX_RIGHT ) then
               xpos = MAX_RIGHT
          elseif ( xpos < MAX_LEFT ) then
               xpos = MAX_LEFT
          end
          ypos = nil
     end
     self:SetButtonXY( xpos, ypos )
end

-- AutoProfitX:OnEnterButton( self )
-- =========================================================
function AutoProfitX:OnEnterButton( self )
     GameTooltip:SetOwner( self, "ANCHOR_RIGHT" )
     GameTooltip:SetText( L["APX_BUTTON_SELLJUNK"] )
     local profit = AutoProfitX:GetProfit()
     if ( AutoProfitX.db.char.buttonSpin == 2 ) then AutoProfitX:UpdateButton() end
     if ( profit > 0 ) then
          if ( AutoProfitX.db.char.buttonSpin == 1 ) then AutoProfitX:UpdateButton() end
          SetTooltipMoney( GameTooltip, profit )
     else
          GameTooltip:AddLine( L["APX_BUTTON_NOJUNK"], 1.0, 1.0, 1.0, 1 )
     end
     GameTooltip:Show()
end

-- AutoProfitX:OnLeaveButton()
-- =========================================================
function AutoProfitX:OnLeaveButton()
     GameTooltip:Hide()
     if ( AutoProfitX.db.char.buttonSpin ~= 3 ) then AutoProfitX:UpdateButton( "stop" ) end
end

-- AutoProfitX:OnClickButton()
-- =========================================================
function AutoProfitX:OnClickButton()
     local profit = AutoProfitX:GetProfit()
     if ( profit > 0 ) then
          GameTooltip:Hide()
          AutoProfitX:SellJunk()
          if ( AutoProfitX.db.char.showTotal ) then
               if ( AutoProfitX.db.char.fancyprofits ) then 
                    self:Print( "print", format( L["APX_BUTTON_PRINT_PROFIT"], AutoProfitX:GetMoneyString( profit ) ) );
               else
                    self:Print( "print", format( L["APX_BUTTON_PRINT_PROFIT"], AutoProfitX:CopperToGold( profit ) ) );
               end
          end
	  --- this is going to be removed SEE - AutoProfitX:ButtonState()
          if ( AutoProfitX.db.char.buttonSpin ~= 2 ) then
               AutoProfitX:UpdateButton( "stop" )
          end
     end
end

-- AutoProfitX:OnShowButton()
-- =========================================================
function AutoProfitX:ButtonState()
     -- Needed for OnEnterButton
     if ( AutoProfitX.db.char.buttonSpin == 1 ) then
          AutoProfitX:UpdateButton()
     end
     -- Needed for OnLeaveButton
     if ( AutoProfitX.db.char.buttonSpin ~= 3 ) then
          AutoProfitX:UpdateButton( "stop" )
     end
     -- Needed for OnClick
     if ( AutoProfitX.db.char.buttonSpin ~= 2 ) then
          AutoProfitX:UpdateButton( "stop" )
     end
     -- Needed for OnShowButton
     if ( AutoProfitX.db.char.buttonSpin == 3 and AutoProfitX:GetProfit() > 0 ) then
          AutoProfitX:UpdateButton()
     end
end

-- AutoProfitX:OnShowButton()
-- =========================================================
function AutoProfitX:OnShowButton( self )
     if ( AutoProfitX.db.char.autoSell ) then
          self:Hide()
     end
     if ( AutoProfitX.db.char.buttonSpin == 3 and AutoProfitX:GetProfit() > 0 ) then
          AutoProfitX:UpdateButton()
     end
end

-- AutoProfitX:SetButtonXY( buttonXpos, buttonYpos )
-- =========================================================
function AutoProfitX:SetButtonXY( X, Y )
     X = tonumber( X )
     Y = tonumber( Y )
     if ( X ) then
          AutoProfitX.db.char.buttonXpos = X
          AutoProfitX:Print( "bug", "X = "..X )
     end
     if ( Y ) then
          AutoProfitX.db.char.buttonYpos = Y
          AutoProfitX:Print( "bug", "Y = "..Y )
     end
     AutoProfitX_SellButton:SetPoint( "TOPRIGHT", "MerchantFrame", "TOPRIGHT", AutoProfitX.db.char.buttonXpos, AutoProfitX.db.char.buttonYpos );
end

-- AutoProfitX:AddTooltipMoney( s )
-- =========================================================
function AutoProfitX:AddTooltipMoney( s )
     if ( s ) then
          totalProfit = totalProfit + s
     end
end

-- AutoProfitX:UpdateButton( action, spinRate )
-- =========================================================
function AutoProfitX:UpdateButton( action, spinRate )
     if ( action == "stop" ) then
          AutoProfitX_SellButton_TreasureModel.rotRate = 0;
     else
          spinRate = tonumber( spinRate )
          if ( not spinRate ) then
               spinRate = AutoProfitX.db.char.spinRate
          end
          AutoProfitX_SellButton_TreasureModel.rotRate = spinRate;
     end

end

-- AutoProfitX:Print( action, msg )
-- =========================================================
function AutoProfitX:Print( action, msg )
     if ( action == "bug" and AutoProfitX.db.global.debug.state ) then
          DEFAULT_CHAT_FRAME:AddMessage( format( L["APX_PRINT_DEBUGLINE"], msg) );
     elseif ( action == "print" ) then
          DEFAULT_CHAT_FRAME:AddMessage( format( L["APX_PRINT_PRINTLINE"], msg ) );
     elseif ( action == "error" ) then
          DEFAULT_CHAT_FRAME:AddMessage( format( L["APX_PRINT_ERRORLINE"], msg ) );
     end
end

-- AutoProfitX:ButtonClick( action )
-- =========================================================
function AutoProfitX:ButtonClick( action )
     local CheckS, CheckA = IsShiftKeyDown(), IsAltKeyDown();
     if ( action == "config" ) then
          InterfaceOptionsFrame_OpenToCategory( AutoProfitX.ConfigFrames.AutoProfitX );
     elseif ( action == "list" ) then
          if ( AutoProfitX.db.global.debug.state and CheckA and CheckS ) then ReloadUI(); end
          if ( not AutoProfitXFrame:IsVisible() ) then
               AutoProfitX:Print( "bug", "Show AutoProfitX Frame");
               AutoProfitXFrame:Show();
          else
               AutoProfitX:Print( "bug", "Hideing AutoProfitX Frame");
               AutoProfitXFrame:Hide();
          end
     end
end

-- AutoProfitX:GetMoneyString( money )
-- =========================================================
function AutoProfitX:GetMoneyString( money )
     local gold = floor( money / 10000 )
     local silver = floor( ( money - gold * 10000 ) / 100 )
     local copper = mod( money, 100 )
     if ( gold > 0 ) then
          return format( GOLD_AMOUNT_TEXTURE.." "..SILVER_AMOUNT_TEXTURE.." "..COPPER_AMOUNT_TEXTURE, gold, 0, 0, silver, 0, 0, copper, 0, 0 )
     elseif ( silver > 0 ) then
          return format( SILVER_AMOUNT_TEXTURE.." "..COPPER_AMOUNT_TEXTURE, silver, 0, 0, copper, 0, 0 )
     else
          return format( COPPER_AMOUNT_TEXTURE, copper, 0, 0 )
     end
end

-- AutoProfitX:CopperToGold( copper, showGold )
-- =========================================================
function AutoProfitX:CopperToGold( copper )
     local strValue = ""
     local val
     val = math.floor( copper / COPPER_PER_GOLD )
     copper = mod( copper , COPPER_PER_GOLD )
     if ( val > 0 ) then
          strValue = val .. L["g "]
     end
     val = math.floor(copper/COPPER_PER_SILVER)
     copper = mod(copper,COPPER_PER_SILVER)
     if ( val > 0 or strValue ~= "" ) then
          strValue = strValue .. val .. L["s "]
     end
     return strValue .. copper .. L["c"]
end

-- AutoProfitX:OpenAllBags( Close )
-- =========================================================
function AutoProfitX:OpenAllBags( Close )
     for bag = 0, 4 do
          if ( not Close ) then
               if ( not IsBagOpen( bag ) ) then
                    AutoProfitX.db.global.bag[bag] = true;
                    OpenBag(bag);
               end
          elseif AutoProfitX.db.global.bag[bag] then
                    AutoProfitX.db.global.bag[bag] = false;
                    CloseBag(bag);
          end
     end
end

-- AutoProfitX:otherBag()
-- =========================================================
function AutoProfitX:otherBag()
     local otherBag = false;
     if ( IsAddOnLoaded("BaudBag") ) then otherBag = true; end
     if ( IsAddOnLoaded("ArkInventory") ) then otherBag = true; end
     if ( IsAddOnLoaded("Bagnon") ) then otherBag = true; end
     if ( IsAddOnLoaded("OneBag3") ) then otherBag = true; end
     return otherBag
end

-- AutoProfitX:tdeepcopy( from )
-- =========================================================
local function tdeepcopy( from )
     if ( type( from ) == "table" ) then
          local t = {}
          for k, v in pairs( from ) do
               if ( type(v) == "table" ) then
                    t[k] = tdeepcopy( v )
               else
                    t[k] = v
               end
          end
          return t
     end
     return from
end

-- AutoProfitX:ExceptionsFrame()
-- =========================================================
function AutoProfitX:ExceptionsFrame()

     local apx = AceGUI:Create("Window");
     AutoProfitXFrame = apx.frame;
     apx:SetTitle( L["APX_PRINT_PRINTLINE"]:format( Version ) );
     apx:SetWidth( 435 );
     apx:SetHeight( 310 );

     apx.sizer_se:Hide();
     apx.sizer_e:Hide();
     apx.sizer_s:Hide();
     apx:SetLayout( "Flow" );

     local cHeader = AceGUI:Create("Heading");
     cHeader:SetText( L["APX_FRAME_EXCEP"]:format( L["APX_FRAME_CLEAR"] ) );
     cHeader:SetFullWidth( "isFull" );

     local cGlobal = AceGUI:Create("Button");
     cGlobal:SetText( L["APX_FRAME_EXCEP"]:format( L["APX_FRAME_GLOBAL"] ) );
     cGlobal:SetFullWidth();
     cGlobal:SetCallback( "OnClick",
          function() 
               wipe( AutoProfitX.db.global.geList )
               AutoProfitX:Print( "print", format( L["APX_FRAME_RESET"], L["APX_FRAME_GLOBAL"] ) );
          end
     )

     local cCharacter = AceGUI:Create("Button");
     cCharacter:SetText( L["APX_FRAME_EXCEP"]:format( L["APX_FRAME_LOCAL"] ) );
     cCharacter:SetFullWidth();
     cCharacter:SetCallback( "OnClick",
          function()
               wipe( AutoProfitX.db.char.leList );
               AutoProfitX:Print( "print", format( L["APX_FRAME_RESET"], L["APX_FRAME_LOCAL"] ) );
          end
     )

     local pgHeader = AceGUI:Create("Heading");
     pgHeader:SetText( L["APX_FRAME_PRINT"] );
     pgHeader:SetFullWidth("isFull");

     local pGlobal = AceGUI:Create("Button");
     pGlobal:SetText( L["APX_FRAME_PRINT_GROUP"]:format( L["APX_FRAME_GLOBAL"] ) );
     pGlobal:SetFullWidth();
     pGlobal:SetCallback( "OnClick",
          function()
               -- wipe( AutoProfitX.db.char.leList );
               -- AutoProfitX:Print( "print", format( L["APX_FRAME_RESET"], L["APX_FRAME_LOCAL"] ) );
               AutoProfitX:ListExceptions( "Global" );
          end
     )
     local pLocal = AceGUI:Create("Button");
     pLocal:SetText( L["APX_FRAME_PRINT_GROUP"]:format( L["APX_FRAME_LOCAL"] ) );
     pLocal:SetFullWidth();
     pLocal:SetCallback( "OnClick",
          function()
               -- wipe( AutoProfitX.db.char.leList );
	       -- AutoProfitX:Print( "print", format( L["APX_FRAME_RESET"], L["APX_FRAME_LOCAL"] ) );
               AutoProfitX:ListExceptions( "Local" );
          end
     )

     local gHeader = AceGUI:Create("Heading");
     gHeader:SetText( L["APX_FRAME_EXCEP"]:format( L["APX_FRAME_GLOBAL"] ) );
     gHeader:SetFullWidth( "isFull" );

     local gLabel = AceGUI:Create("Label");
     gLabel:SetText( L["APX_FRAME_DRAGDROP"] );
     gLabel:SetFullWidth( "isFull" );

     local aGlobal = AceGUI:Create("EditBox");
     aGlobal:DisableButton(true);
     aGlobal:SetLabel( L["APX_FRAME_ADDGLOBAL"] );
     aGlobal:SetCallback( "OnEnterPressed", function( widget, event, text )
               AutoProfitX:idUtils( "idAdd", "global", text );
               aGlobal:SetText("");
          end
     )

     local rGlobal = AceGUI:Create("EditBox");
     rGlobal:DisableButton(true);
     rGlobal:SetLabel( L["APX_FRAME_REMOVEGLOBAL"] );
     rGlobal:SetCallback( "OnEnterPressed", function( widget, event, text )
               AutoProfitX:idUtils( "idRemove", "global", text );
               rGlobal:SetText("");
          end
     )

     local chHeader = AceGUI:Create("Heading");
     chHeader:SetText( L["APX_FRAME_EXCEP"]:format( L["APX_FRAME_LOCAL"] ) );
     chHeader:SetFullWidth( "isFull" );

     local cLabel = AceGUI:Create("Label");
     cLabel:SetText( L["APX_FRAME_DRAGDROP"] );
     cLabel:SetFullWidth( "isFull" );

     local aCharacter = AceGUI:Create("EditBox");
     aCharacter:DisableButton(true);
     aCharacter:SetLabel( L["APX_FRAME_ADDLOCAL"] );
     aCharacter:SetCallback( "OnEnterPressed", function( widget, event, text )
               AutoProfitX:idUtils( "idAdd", "local", text );
               aCharacter:SetText("");
          end
     )

     local rCharacter = AceGUI:Create("EditBox");
     rCharacter:DisableButton(true);
     rCharacter:SetLabel( L["APX_FRAME_REMOVELOCAL"] );
     rCharacter:SetCallback( "OnEnterPressed", function( widget, event, text )
               AutoProfitX:idUtils( "idRemove", "local", text );
               rCharacter:SetText("");
          end
     )

     apx:AddChildren( cHeader, cGlobal, cCharacter, pgHeader, pGlobal, pLocal, gHeader, gLabel, aGlobal, rGlobal, chHeader, cLabel, aCharacter, rCharacter );
     AutoProfitXFrame:Hide();
end